import {useErrorStore} from '@/stores/error';
import type {
  AddViewConfig,
  FormValues,
  WorkGroupDetailResponseModel,
} from '../models';
import {fetchAuthenticated} from '../retrieveData';
import {getDetailResponseModelMocked, getError, getSaveError, type SaveError} from '@/helpers/dataAccess';
import i18next from 'i18next';


const fetchWorkGroupAddViewConfigMocked = (): AddViewConfig => {
  const config: AddViewConfig = {
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'general__basicSettings',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'displayName',
                    label: 'Display name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'Name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UComboBox',
                  props: {
                    name: 'school',
                    label: 'School',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'Schule 1',
                        value: 'school1',
                      },
                      {
                        label: 'Schule 2',
                        value: 'school2',
                      },
                    ],
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'divisStartdate',
                    label: 'Creation date',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputDate',
                  props: {
                    name: 'divisEnddate',
                    label: 'Expiry date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
        ],
      },
    ],
  };
  console.debug('mock config', config);
  return config;
};
export const fetchWorkGroupAddViewConfig = async(): Promise<AddViewConfig> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchWorkGroupAddViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchWorkGroupAddViewConfigMocked());
      }, 1000);
    });
  }
  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/pageconf/addView');
  const error = await getError(answer);
  if (error) {
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchListViewConfig.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
    });
    return Promise.reject(answer);
  }

  return await answer.json();
};

export const createWorkGroup = async(values: FormValues): Promise<SaveError | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('createWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(null);
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/workgroup', {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });
  return await getSaveError(answer, 'createWorkgroup');
};

const fetchWorkGroupMocked = (): WorkGroupDetailResponseModel => {
  const workgroup: WorkGroupDetailResponseModel = getDetailResponseModelMocked();
  console.debug('mock workgroup', workgroup);
  return workgroup;
};

export const fetchWorkGroup = async(workgroup: string): Promise<WorkGroupDetailResponseModel | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchWorkGroupMocked());
      }, 1000);
    });
  }

  // TODO adjust url again when issue https://git.knut.univention.de/univention/ucsschool-components/ui-groups/-/issues/47 is done
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/workgroup/${workgroup}`);
  if (answer.status === 404) {
    return null;
  }
  const error = await getError(answer);
  if (error) {
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchGroup.error.title', {context: 'workgroup'});
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};

export const updateWorkGroup = async(url: string, values: FormValues): Promise<SaveError | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('updateWorkGroup. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    // TODO have the fail case in some way too
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(null);
      }, 1000);
    });
  }

  const fixedProtocolUrl = url.replace(/https?:/, location.protocol);
  const answer = await fetchAuthenticated(`${fixedProtocolUrl}`, {
    method: 'PATCH',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });
  return await getSaveError(answer, 'createWorkgroup');
};
